!=======================================================================================
!
!Copyright 2013 Ibero Rubik
!
!This file is part of Ibero Solver 2x2x2 LBL.
!
!Ibero Solver 2x2x2 LBL is free software: you can redistribute it and/or modify
!it under the terms of the GNU General Public License as published by
!the Free Software Foundation, either version 3 of the License, or
!any later version.
!
!Ibero Solver 2x2x2 LBL is distributed in the hope that it will be useful,
!but WITHOUT ANY WARRANTY; without even the implied warranty of
!MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!GNU General Public License for more details.
!
!You should have received a copy of the GNU General Public License
!along with Ibero Solver 2x2x2 LBL.  If not, see <http://www.gnu.org/licenses/>.
!
!=======================================================================================


module module_1

	use module_3

	implicit none
	
	contains
	
	!______________________________________________________________________
	
		subroutine colour_pattern (MU, MF, MR, MB, ML, MD)
		
			character (len=10), intent(inout)	:: MU(2,2), MF(2,2), MR(2,2), MB(2,2), ML(2,2), MD(2,2)
			
			integer				:: i, j
			character (len=10)	:: supermatrix (2,12)
			
			supermatrix (:,1:2) = MU
			supermatrix (:,3:4) = MF
			supermatrix (:,5:6) = MR
			supermatrix (:,7:8) = MB
			supermatrix (:,9:10) = ML
			supermatrix (:,11:12) = MD
			
			do i = 1, 2
			
				do j = 1, 12
				
					if (supermatrix(i,j) == 'purple') then
					
						supermatrix (i,j) = 'orange'
						
					else if (supermatrix(i,j) == 'black') then
					
						supermatrix (i,j) = 'white'
					
					end if
					
				end do
				
			end do
			
			MU = supermatrix (:,1:2)
			MF = supermatrix (:,3:4)
			MR = supermatrix (:,5:6)
			MB = supermatrix (:,7:8)
			ML = supermatrix (:,9:10)
			MD = supermatrix (:,11:12)
			
			write (*,*) 'After the pattern detection, the faces are:'
	
			write(*,*) '----------------------------'
			write(*,*) 'U'
			do i = 1, 2
				write(*,*) (MU(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'
			
			write(*,*) '----------------------------'
			write(*,*) 'F'
			do i = 1, 2
				write(*,*) (MF(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'

			write(*,*) '----------------------------'
			write(*,*) 'R'
			do i = 1, 2
				write(*,*) (MR(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'
			
			write(*,*) '----------------------------'
			write(*,*) 'B'
			do i = 1, 2
				write(*,*) (MB(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'
			
			write(*,*) '----------------------------'
			write(*,*) 'L'
			do i = 1, 2
				write(*,*) (ML(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'
			
			write(*,*) '----------------------------'
			write(*,*) 'D'
			do i = 1, 2
				write(*,*) (MD(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'
			
			write(*,*) ' '
			
		end subroutine colour_pattern

	!______________________________________________________________________
	
	subroutine error_detection (MU, MF, MR, MB, ML, MD)
		
			character (len=10), intent(inout)	:: MU(2,2), MF(2,2), MR(2,2), MB(2,2), ML(2,2), MD(2,2)
			
			integer				:: i, j, num(6), u7, error
			character (len=10)	:: supermatrix (2,12)
			
			u7 = 200
			
			open (unit = u7, file = 'program_files/errors.txt', status = 'unknown', action = 'write', iostat = error)
			
			
			supermatrix (:,1:2) = MU
			supermatrix (:,3:4) = MF
			supermatrix (:,5:6) = MR
			supermatrix (:,7:8) = MB
			supermatrix (:,9:10) = ML
			supermatrix (:,11:12) = MD
			
			num = 0
			
			do i = 1, 2
			
				do j = 1, 12
				
					if ((supermatrix(i,j) /= 'white').and.(supermatrix(i,j) /= 'red').and.(supermatrix(i,j) /= 'blue')&
					&.and.(supermatrix(i,j) /= 'orange').and.(supermatrix(i,j) /= 'green').and.(supermatrix(i,j) /= 'yellow')) then
						
						write(u7,*) 'Some colour has been badly written'
						write(*,*) 'Some colour has been badly written'
						stop
					
					end if
					
					select case (supermatrix(i,j))
					
						case ('white')
							num(1) = num(1) + 1
						case ('red')
							num(2) = num(2) + 1	
						case ('blue')
							num(3) = num(3) + 1	
						case ('orange')
							num(4) = num(4) + 1	
						case ('green')
							num(5) = num(5) + 1
						case ('yellow')
							num(6) = num(6) + 1
							
					end select
					
				end do
				
			end do
			
			if (any (num /= 4)) then
			
				write(u7,*) 'Some colour has been written more than four times'
				write(*,*) 'Some colour has been written more than four times'
				stop
				
			end if
	
	end subroutine error_detection
	
	
	!______________________________________________________________________
	
		subroutine step_detection (MU, MF, MR, MB, ML, MD, n, step)
		
			character (len=10), intent(inout)	:: MU(2,2), MF(2,2), MR(2,2), MB(2,2), ML(2,2), MD(2,2)
			integer, intent(out)				:: n, step
			
			integer								:: i, j, u1, u7, error
			character(len=10)					:: supermatrix(2,12)
			
			
			n	= 0
			u1	= 201
			u7	= 202
			
			open (unit = u7, file = "program_files/errors.txt", status = 'unknown', action = 'write', iostat = error)
			
			!------------------------------------------------------------------------------------
			!1st part; detection of the most solved face to place it on the top
			!------------------------------------------------------------------------------------
			
			open (unit = u1, file = 'output_files/movements_step_0.txt', status = 'replace', action = 'write', iostat = error)
			if (error > 0) then
				write(u7,*) 'subroutine step_detector: file "output_files/movements_step_0.txt" did not open correctly'
				stop 'subroutine step_detector: file "output_files/movements_step_0.txt" did not open correctly'
			end if
			
			if ((MU(1,1)==MU(1,2)).and.(MU(1,1)==MU(2,1)).and.(MU(1,1)==MU(2,2))) then
			
				n = 4
				
			else if ((MF(1,1)==MF(1,2)).and.(MF(1,1)==MF(2,1)).and.(MF(1,1)==MF(2,2))) then
			
				n = 4
				call x (MU=MU, MF=MF, MR=MR, MB=MB, ML=ML, MD=MD)
				write(u1,*) "x"
				
			else if ((MR(1,1)==MR(1,2)).and.(MR(1,1)==MR(2,1)).and.(MR(1,1)==MR(2,2))) then
			
				n = 4
				call z_prime (MU=MU, MF=MF, MR=MR, MB=MB, ML=ML, MD=MD)
				write(u1,*) "z'"
				
			else if ((MB(1,1)==MB(1,2)).and.(MB(1,1)==MB(2,1)).and.(MB(1,1)==MB(2,2))) then
			
				n = 4
				call x_prime (MU=MU, MF=MF, MR=MR, MB=MB, ML=ML, MD=MD)
				write(u1,*) "x'"
				
			else if ((ML(1,1)==ML(1,2)).and.(ML(1,1)==ML(2,1)).and.(ML(1,1)==ML(2,2))) then
			
				n = 4
				call z (MU=MU, MF=MF, MR=MR, MB=MB, ML=ML, MD=MD)
				write(u1,*) "z"
				
			else if ((MD(1,1)==MD(1,2)).and.(MD(1,1)==MD(2,1)).and.(MD(1,1)==MD(2,2))) then
			
				n = 4
				call x_2 (MU=MU, MF=MF, MR=MR, MB=MB, ML=ML, MD=MD)
				write(u1,*) "x2"
				
			else if (((MU(1,1)==MU(1,2)).and.(MU(1,1)==MU(2,1)))&
				&.or.((MU(1,1)==MU(1,2)).and.(MU(1,1)==MU(2,2)))&
				&.or.((MU(1,1)==MU(2,1)).and.(MU(1,1)==MU(2,2)))&
				&.or.((MU(1,2)==MU(2,1)).and.(MU(1,2)==MU(2,2)))) then
				
				n = 3
				
			else if (((MF(1,1)==MF(1,2)).and.(MF(1,1)==MF(2,1)))&
				&.or.((MF(1,1)==MF(1,2)).and.(MF(1,1)==MF(2,2)))&
				&.or.((MF(1,1)==MF(2,1)).and.(MF(1,1)==MF(2,2)))&
				&.or.((MF(1,2)==MF(2,1)).and.(MF(1,2)==MF(2,2)))) then
				
				n = 3
				call x (MU=MU, MF=MF, MR=MR, MB=MB, ML=ML, MD=MD)
				write(u1,*) "x"
				
			else if (((MR(1,1)==MR(1,2)).and.(MR(1,1)==MR(2,1)))&
				&.or.((MR(1,1)==MR(1,2)).and.(MR(1,1)==MR(2,2)))&
				&.or.((MR(1,1)==MR(2,1)).and.(MR(1,1)==MR(2,2)))&
				&.or.((MR(1,2)==MR(2,1)).and.(MR(1,2)==MR(2,2)))) then
				
				n = 3
				call z_prime (MU=MU, MF=MF, MR=MR, MB=MB, ML=ML, MD=MD)
				write(u1,*) "z'"
				
			else if (((MB(1,1)==MB(1,2)).and.(MB(1,1)==MB(2,1)))&
				&.or.((MB(1,1)==MB(1,2)).and.(MB(1,1)==MB(2,2)))&
				&.or.((MB(1,1)==MB(2,1)).and.(MB(1,1)==MB(2,2)))&
				&.or.((MB(1,2)==MB(2,1)).and.(MB(1,2)==MB(2,2)))) then
				
				n = 3
				call x_prime (MU=MU, MF=MF, MR=MR, MB=MB, ML=ML, MD=MD)
				write(u1,*) "x'"
				
			else if (((ML(1,1)==ML(1,2)).and.(ML(1,1)==ML(2,1)))&
				&.or.((ML(1,1)==ML(1,2)).and.(ML(1,1)==ML(2,2)))&
				&.or.((ML(1,1)==ML(2,1)).and.(ML(1,1)==ML(2,2)))&
				&.or.((ML(1,2)==ML(2,1)).and.(ML(1,2)==ML(2,2)))) then
				
				n = 3
				call z (MU=MU, MF=MF, MR=MR, MB=MB, ML=ML, MD=MD)
				write(u1,*) "z"
				
			else if (((MD(1,1)==MD(1,2)).and.(MD(1,1)==MD(2,1)))&
				&.or.((MD(1,1)==MD(1,2)).and.(MD(1,1)==MD(2,2)))&
				&.or.((MD(1,1)==MD(2,1)).and.(MD(1,1)==MD(2,2)))&
				&.or.((MD(1,2)==MD(2,1)).and.(MD(1,2)==MD(2,2)))) then
				
				n = 3
				call x_2 (MU=MU, MF=MF, MR=MR, MB=MB, ML=ML, MD=MD)
				write(u1,*) "x2"
				
			else			!Groups of 3 or 4 matched colours not found; the cube will turn to place some white on the top
				
				if (any(MU == 'white')) then
					n = 1
				else if (any(MF == 'white')) then
					n = 1
					call x (MU=MU, MF=MF, MR=MR, MB=MB, ML=ML, MD=MD)
					write(u1,*) "x"
				else if (any(MR == 'white')) then
					n = 1
					call z_prime (MU=MU, MF=MF, MR=MR, MB=MB, ML=ML, MD=MD)
					write(u1,*) "z'"
				else if (any(MB == 'white')) then
					n = 1
					call x_prime (MU=MU, MF=MF, MR=MR, MB=MB, ML=ML, MD=MD)
					write(u1,*) "x'"
				else if (any(ML == 'white')) then
					n = 1
					call z (MU=MU, MF=MF, MR=MR, MB=MB, ML=ML, MD=MD)
					write(u1,*) "z"
				end if

			end if
			
	
			
			!------------------------------------------------------------------------------------
			!2nd part; calculations about how solved the cube is
			!------------------------------------------------------------------------------------
				
			if (n == 4) then
			
				if ((MF(1,1) == MF(1,2)).and.(MR(1,1) == MR(1,2)).and.(MB(1,1) == MB(1,2)).and.(ML(1,1) == ML(1,2))) then
				
					if ( (MF(1,1)==MF(2,1)).and.(MF(1,2)==MF(2,2)).and.(MR(1,1)==MR(2,1)).and.(MR(1,2)==MR(2,2))&
					&.and.(MB(1,1)==MB(2,1)).and.(MB(1,2)==MB(2,2)).and.(ML(1,1)==ML(2,1)).and.(ML(1,2)==ML(2,2)) ) then
					
						step = 4
						
					else if ( (MD(1,1)==MD(1,2)).and.(MD(1,1)==MD(2,1)).and.(MD(1,1)==MD(2,2)) ) then
					
						step = 3
						
					else
				
						step = 2
						
					end if
					
				else
					step = 1
					
				end if
				
			else
			
				step = 1
				
			end if
			
			write (*,*) 'After turning the cube and detecting the step, the faces are:'
	
			write(*,*) '----------------------------'
			write(*,*) 'U'
			do i = 1, 2
				write(*,*) (MU(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'
			
			write(*,*) '----------------------------'
			write(*,*) 'F'
			do i = 1, 2
				write(*,*) (MF(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'

			write(*,*) '----------------------------'
			write(*,*) 'R'
			do i = 1, 2
				write(*,*) (MR(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'
			
			write(*,*) '----------------------------'
			write(*,*) 'B'
			do i = 1, 2
				write(*,*) (MB(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'
			
			write(*,*) '----------------------------'
			write(*,*) 'L'
			do i = 1, 2
				write(*,*) (ML(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'
			
			write(*,*) '----------------------------'
			write(*,*) 'D'
			do i = 1, 2
				write(*,*) (MD(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'
			
			write(*,*) ' '
			
			
			
		end subroutine step_detection
		
	!______________________________________________________________________		
		
		subroutine colour_change (MU, MF, MR, MB, ML, MD, n)
		
			character (len=10), intent(inout)	:: MU(2,2), MF(2,2), MR(2,2), MB(2,2), ML(2,2), MD(2,2)
			integer, intent(in)					:: n
			
			integer								:: i, j
			character(len=10)					:: supermatrix(2,12), top_colour
		
			top_colour = 'white'
			
			if (n == 4) then

				top_colour	= MU(1,1)
				
			else if (n == 3) then
			
				if (MU(1,1) == MU(1,2)) then
					top_colour = MU(1,1)
				else
					top_colour = MU(2,1)
				end if
				
			end if
			
			supermatrix (:,1:2) = MU
			supermatrix (:,3:4) = MF
			supermatrix (:,5:6) = MR
			supermatrix (:,7:8) = MB
			supermatrix (:,9:10) = ML
			supermatrix (:,11:12) = MD
			
			select case (top_colour)
			
				case ('red')
				
				do i = 1, 2				
					do j = 1, 12					
						if (supermatrix(i,j) == 'red') then						
							supermatrix(i,j) = 'white'							
						else if (supermatrix(i,j) == 'white') then						
							supermatrix(i,j) = 'red'
						else if (supermatrix(i,j) == 'green') then						
							supermatrix(i,j) = 'blue'
						else if (supermatrix(i,j) == 'yellow') then						
							supermatrix(i,j) = 'orange'
						else if (supermatrix(i,j) == 'blue') then						
							supermatrix(i,j) = 'green'
						else if (supermatrix(i,j) == 'orange') then						
							supermatrix(i,j) = 'yellow'
						end if
					end do
				end do
				
				case ('blue')
				
				do i = 1, 2				
					do j = 1, 12					
						if (supermatrix(i,j) == 'blue') then						
							supermatrix(i,j) = 'white'							
						else if (supermatrix(i,j) == 'white') then						
							supermatrix(i,j) = 'red'
						else if (supermatrix(i,j) == 'red') then						
							supermatrix(i,j) = 'blue'
						else if (supermatrix(i,j) == 'yellow') then						
							supermatrix(i,j) = 'orange'
						else if (supermatrix(i,j) == 'orange') then						
							supermatrix(i,j) = 'green'
						else if (supermatrix(i,j) == 'orange') then						
							supermatrix(i,j) = 'yellow'
						end if
					end do
				end do
				
				case ('orange')
				
				do i = 1, 2				
					do j = 1, 12					
						if (supermatrix(i,j) == 'orange') then						
							supermatrix(i,j) = 'white'							
						else if (supermatrix(i,j) == 'white') then						
							supermatrix(i,j) = 'red'
						else if (supermatrix(i,j) == 'blue') then						
							supermatrix(i,j) = 'blue'
						else if (supermatrix(i,j) == 'yellow') then						
							supermatrix(i,j) = 'orange'
						else if (supermatrix(i,j) == 'green') then						
							supermatrix(i,j) = 'green'
						else if (supermatrix(i,j) == 'red') then						
							supermatrix(i,j) = 'yellow'
						end if
					end do
				end do
				
				case ('green')
				
				do i = 1, 2				
					do j = 1, 12					
						if (supermatrix(i,j) == 'green') then						
							supermatrix(i,j) = 'white'							
						else if (supermatrix(i,j) == 'white') then						
							supermatrix(i,j) = 'red'
						else if (supermatrix(i,j) == 'orange') then						
							supermatrix(i,j) = 'blue'
						else if (supermatrix(i,j) == 'yellow') then						
							supermatrix(i,j) = 'orange'
						else if (supermatrix(i,j) == 'red') then						
							supermatrix(i,j) = 'green'
						else if (supermatrix(i,j) == 'blue') then						
							supermatrix(i,j) = 'yellow'
						end if
					end do
				end do
			
				case ('yellow')
				
				do i = 1, 2				
					do j = 1, 12					
						if (supermatrix(i,j) == 'yellow') then						
							supermatrix(i,j) = 'white'							
						else if (supermatrix(i,j) == 'red') then						
							supermatrix(i,j) = 'red'
						else if (supermatrix(i,j) == 'green') then						
							supermatrix(i,j) = 'blue'
						else if (supermatrix(i,j) == 'orange') then						
							supermatrix(i,j) = 'orange'
						else if (supermatrix(i,j) == 'blue') then						
							supermatrix(i,j) = 'green'
						else if (supermatrix(i,j) == 'white') then						
							supermatrix(i,j) = 'yellow'
						end if
					end do
				end do
				
			end select
				
			MU = supermatrix (:,1:2)
			MF = supermatrix (:,3:4)
			MR = supermatrix (:,5:6)
			MB = supermatrix (:,7:8)
			ML = supermatrix (:,9:10)
			MD = supermatrix (:,11:12)
			
			write (*,*) 'After the internal colour change, the faces are:'
	
			write(*,*) '----------------------------'
			write(*,*) 'U'
			do i = 1, 2
				write(*,*) (MU(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'
			
			write(*,*) '----------------------------'
			write(*,*) 'F'
			do i = 1, 2
				write(*,*) (MF(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'

			write(*,*) '----------------------------'
			write(*,*) 'R'
			do i = 1, 2
				write(*,*) (MR(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'
			
			write(*,*) '----------------------------'
			write(*,*) 'B'
			do i = 1, 2
				write(*,*) (MB(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'
			
			write(*,*) '----------------------------'
			write(*,*) 'L'
			do i = 1, 2
				write(*,*) (ML(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'
			
			write(*,*) '----------------------------'
			write(*,*) 'D'
			do i = 1, 2
				write(*,*) (MD(i,j), j = 1,2)
			end do
			write(*,*) '----------------------------'
			
			write(*,*) ' '
	
		end subroutine colour_change
		
	!______________________________________________________________________
		
		subroutine join_files
		
			integer 			:: u0, u1, u2, u3, u4, u7, error
			character(len=10)	:: movement
			
			u0 = 101
			u1 = 102
			u2 = 103
			u3 = 104
			u4 = 201
			u7 = 202
			
			open (unit = u7, file = "program_files/errors.txt", status = 'unknown', action = 'write', iostat = error)
			
			open (unit = u0, file = 'output_files/movements_step_0.txt', status = 'unknown', action = 'read', iostat = error)
			if (error > 0) then
				write(u7,*) 'subroutine join_files: file "output_files/movements_step_0.txt" did not open correctly'
				stop 'subroutine join_files: file "output_files/movements_step_0.txt" did not open correctly'
			end if
			
			open (unit = u1, file = 'output_files/movements_step_1.txt', status = 'unknown', action = 'read', iostat = error)
			if (error > 0) then
				write(u7,*) 'subroutine join_files: file "output_files/movements_step_1.txt" did not open correctly'
				stop 'subroutine join_files: file "output_files/movements_step_1.txt" did not open correctly'
			end if
			
			open (unit = u2, file = 'output_files/movements_step_2.txt', status = 'unknown', action = 'read', iostat = error)
			if (error > 0) then
				write(u7,*) 'subroutine join_files: file "output_files/movements_step_2.txt" did not open correctly'
				stop 'subroutine join_files: file "output_files/movements_step_2.txt" did not open correctly'
			end if
			
			open (unit = u3, file = 'output_files/movements_step_3.txt', status = 'unknown', action = 'read', iostat = error)
			if (error > 0) then
				write(u7,*) 'subroutine join_files: file "output_files/movements_step_3.txt" did not open correctly'
				stop 'subroutine join_files: file "output_files/movements_step_3.txt" did not open correctly'
			end if
			
			open (unit = u4, file = 'output_files/movements.txt', status = 'replace', action = 'write', iostat = error)
			if (error > 0) then
				write(u7,*) 'subroutine join_files: file "output_files/movements.txt" did not open correctly'
				stop 'subroutine join_files: file "output_files/movements.txt" did not open correctly'
			end if
			
			do
			
				read(u0,*,iostat=error) movement
				if (error < 0) exit
				write(u4,*) movement
				
			end do
			
			do
			
				read(u1,*,iostat=error) movement
				if (error < 0) exit
				write(u4,*) movement
				
			end do
			
			do
			
				read(u2,*,iostat=error) movement
				if (error < 0) exit
				write(u4,*) movement
				
			end do
			
			do
			
				read(u3,*,iostat=error) movement
				if (error < 0) exit
				write(u4,*) movement
				
			end do
			
			
			close (u4)
				
			
			
		
		end subroutine join_files
		
	!______________________________________________________________________		
		
		subroutine movement_reducer_2x2x2 (input_file_name, output_file_name, reduction_number)
		
			character(len=*), intent(in)	:: input_file_name, output_file_name
			integer, intent(out)			:: reduction_number
			
			integer							:: i, j, k, u1, u2, u7, error, n
			character(len=2), allocatable	:: v(:), w(:), p1, p2
			character(len=15)				:: result
			
			u1 = 100
			u2 = 201
			u7 = 202
			
			
			
			open (unit = u7, file = "program_files/errors.txt", status = 'unknown', action = 'write', iostat = error)
			
			open (unit = u1, file = input_file_name, status = 'unknown', action = 'read', iostat = error)
			if (error > 0) then
				write(u7,*) 'subroutine "movement_reducer_2x2x2": file ', input_file_name, &
								&' did not open correctly as a "read" file'
				stop 'subroutine "movement_reducer_2x2x2": file u1 did not open correctly as a "read" file'
			end if
								
			!Vectors allocation
			
			n = 0
			
			rewind(u1)
			
			do
			
				read(u1,*,iostat = error)
				if (error < 0) exit
				n = n + 1
			
			end do
			
			
			allocate (v(n), stat = error)
			if (error > 0) then
				write(u7,*) 'subroutine "movement_reducer_2x2x2": vector "v" was not allocated correctly'
				stop 'subroutine "movement_reducer_2x2x2": vector "v" was not allocated correctly'
			end if
			
			allocate (w(n), stat = error)
			if (error > 0) then
				write(u7,*) 'subroutine "movement_reducer_2x2x2": vector "w" was not allocated correctly'
				stop 'subroutine "movement_reducer_2x2x2": vector "w" was not allocated correctly'
			end if
			
			!Load of the characters in the vector
			
			rewind(u1)
			
			do i = 1, n
			
				read(u1,*) v(i)
				
			end do
			
			close(u1)
			
			!Reduction of characters
			
			i = 1
			j = 1
			
			do
			
				if (i == n) then
				
					w(j) = v(n)
					exit
					
				else if (i > n) then
				
					j = j - 1
					exit
					
				end if
		
				p1 = v(i)
				p2 = v(i+1)
				
				
				select case (p1)
				
					case ("U")
					
						call reduce(p2 = p2, fU = "U ", fU_prime = "U'", fU2 = "U2", fD_prime = "D'", fD = "D ", &
						&fy_prime = "y'", fy = "y ", result = result)	
					
					case ("F")
					
						call reduce(p2 = p2, fU = "F ", fU_prime = "F'", fU2 = "F2", fD_prime = "B'", fD = "B ", &
						&fy_prime = "z'", fy = "z ", result = result)
					
					case ("R")
					
						call reduce(p2 = p2, fU = "R ", fU_prime = "R'", fU2 = "R2", fD_prime = "L'", fD = "L ", &
						&fy_prime = "x'", fy = "x ", result = result)
					
					case ("B")
					
						call reduce(p2 = p2, fU = "B ", fU_prime = "B'", fU2 = "B2", fD_prime = "F'", fD = "F ", &
						&fy_prime = "z ", fy = "z'", result = result)
					
					case ("L")
					
						call reduce(p2 = p2, fU = "L ", fU_prime = "L'", fU2 = "L2", fD_prime = "R'", fD = "R ", &
						&fy_prime = "x ", fy = "x'", result = result)
					
					case ("D")
					
						call reduce(p2 = p2, fU = "D ", fU_prime = "D'", fU2 = "D2", fD_prime = "U'", fD = "U ", &
						&fy_prime = "y ", fy = "y'", result = result)
					
					case ("U'")
					
						call reduce_prime (p2 = p2, fU_prime = "U'", fU = "U ", fU2 = "U2", fD_prime = "D'", fD = "D ", &
						&fy_prime = "y'", fy = "y ", result = result)
					
					case ("F'")
					
						call reduce_prime (p2 = p2, fU_prime = "F'", fU = "F ", fU2 = "F2", fD_prime = "B'", fD = "B ", &
						&fy_prime = "z'", fy = "z ", result = result)
					
					case ("R'")
					
						call reduce_prime (p2 = p2, fU_prime = "R'", fU = "R ", fU2 = "R2", fD_prime = "L'", fD = "L ", &
						&fy_prime = "x'", fy = "x ", result = result)
					
					case ("B'")
					
						call reduce_prime (p2 = p2, fU_prime = "B'", fU = "B ", fU2 = "B2", fD_prime = "F'", fD = "F ", &
						&fy_prime = "z ", fy = "z'", result = result)
					
					case ("L'")
					
						call reduce_prime (p2 = p2, fU_prime = "L'", fU = "L ", fU2 = "L2", fD_prime = "R'", fD = "R ", &
						&fy_prime = "x ", fy = "x'", result = result)
					
					case ("D'")
					
						call reduce_prime (p2 = p2, fU_prime = "D'", fU = "D ", fU2 = "D2", fD_prime = "U'", fD = "U ", &
						&fy_prime = "y ", fy = "y'", result = result)
					
					case ("U2")
					
						call reduce_double (p2 = p2, fU2 = "U2", fU = "U ", fU_prime = "U'", fD2 = "D2", &
						&fy2 = "y2", result = result)
					
					case ("F2")
					
						call reduce_double (p2 = p2, fU2 = "F2", fU = "F ", fU_prime = "F'", fD2 = "B2", &
						&fy2 = "z2", result = result)
					
					case ("R2")
					
						call reduce_double (p2 = p2, fU2 = "R2", fU = "R ", fU_prime = "R'", fD2 = "L2", &
						&fy2 = "x2", result = result)
					
					case ("B2")
					
						call reduce_double (p2 = p2, fU2 = "B2", fU = "B ", fU_prime = "B'", fD2 = "F2", &
						&fy2 = "z2", result = result)
					
					case ("L2")
					
						call reduce_double (p2 = p2, fU2 = "L2", fU = "L ", fU_prime = "L'", fD2 = "R2", &
						&fy2 = "x2", result = result)
					
					case ("D2")
					
						call reduce_double (p2 = p2, fU2 = "D2", fU = "D ", fU_prime = "D'", fD2 = "U2", &
						&fy2 = "y2", result = result)
						
					case ("x")
						
						call reduce_axis (p2 = p2, fx = "x ", fx_prime = "x'", fx2 = "x2", fR = "R "&
						&, fR_prime = "R'", fL = "L ", fL_prime = "L'", result = result)
						
					case ("y")
						
						call reduce_axis (p2 = p2, fx = "y ", fx_prime = "y'", fx2 = "y2", fR = "U "&
						&, fR_prime = "U'", fL = "D ", fL_prime = "D'", result = result)
						
					case ("z")
						
						call reduce_axis (p2 = p2, fx = "z ", fx_prime = "z'", fx2 = "z2", fR = "F "&
						&, fR_prime = "F'", fL = "B ", fL_prime = "B'", result = result)
						
					case ("x'")
						
						call reduce_axis_prime (p2 = p2, fx_prime = "x'", fx = "x ", fx2 = "x2", fR = "R "&
						&, fR_prime = "R'", fL = "L ", fL_prime = "L'", result = result)
						
					case ("y'")
						
						call reduce_axis_prime (p2 = p2, fx_prime = "y'", fx = "y ", fx2 = "y2", fR = "U "&
						&, fR_prime = "U'", fL = "D ", fL_prime = "D'", result = result)
						
					case ("z'")
						
						call reduce_axis_prime (p2 = p2, fx_prime = "z'", fx = "z ", fx2 = "z2", fR = "F "&
						&, fR_prime = "F'", fL = "B ", fL_prime = "B'", result = result)
						
					case ("x2")
						
						call reduce_axis_double (p2 = p2, fx2 = "x2", fx = "x ", fx_prime = "x'"&
						&, fR2 = "R2", fL2 = "L2", result = result)
						
					case ("y2")
						
						call reduce_axis_double (p2 = p2, fx2 = "y2", fx = "y ", fx_prime = "y'"&
						&, fR2 = "U2", fL2 = "D2", result = result)
						
					case ("z2")
						
						call reduce_axis_double (p2 = p2, fx2 = "z2", fx = "z ", fx_prime = "z'"&
						&, fR2 = "F2", fL2 = "B2", result = result)
						
					
				
				end select
				
				
				if (result == "autodeleted") then
				
					i = i + 2
					
					
				else if (result == "null") then
				
					w(j) = p1
					j = j + 1
					i = i + 1
					
				else
				
					w(j) = result
					j = j + 1
					i = i + 2
					
				end if
				
			
			end do
			
			reduction_number = n - j
			
			open (unit = u2, file = output_file_name, status = 'unknown', action = 'write', iostat = error)
			if (error > 0) then
				write(u7,*) 'subroutine "movement_reducer_2x2x2": file ', output_file_name, &
								&' did not open correctly as a "write" file'
				stop 'subroutine "movement_reducer_2x2x2": file u2 did not open correctly as a "write" file'
			end if
			
			do k = 1, j
			
				write(u2,*) trim(w(k))
				
			end do
			
			close (u2)
			
		end subroutine movement_reducer_2x2x2
		
	!______________________________________________________________________		
		
		subroutine reduce (p2, fU, fU_prime, fU2, fD_prime, fD, fy_prime, fy, result)

			character(len=2), intent(in)	:: p2, fU, fU_prime, fU2, fD_prime, fD, fy_prime, fy
			character(len=15), intent(out)	:: result
			
			
			if (p2 == fU) then
			
				result = fU2
				
			else if (p2 == fU_prime) then
			
				result = "autodeleted"
				
			else if (p2 == fU2) then
			
				result = fU_prime
				
			else if (p2 == fD_prime) then
			
				result = fy
				
			else if (p2 == fy_prime) then
			
				result = fD
				
			else
			
				result = "null"
				
			end if
			
		end subroutine reduce
		
	!______________________________________________________________________	
		
		subroutine reduce_prime (p2, fU_prime, fU, fU2, fD_prime, fD, fy_prime, fy, result)

			character(len=2), intent(in)	:: p2, fU, fU_prime, fU2, fD_prime, fD, fy_prime, fy
			character(len=15), intent(out)	:: result
			
			
			if (p2 == fU) then
			
				result = "autodeleted"
				
			else if (p2 == fU_prime) then
			
				result = fU2
				
			else if (p2 == fU2) then
			
				result = fU
				
			else if (p2 == fD) then
			
				result = fy_prime
				
			else if (p2 == fy) then
			
				result = fD_prime
				
			else
			
				result = "null"
				
			end if
			
		end subroutine reduce_prime
		
		!______________________________________________________________________		
		
		subroutine reduce_double (p2, fU2, fU, fU_prime, fD2, fy2, result)

			character(len=2), intent(in)	:: p2, fU, fU_prime, fU2, fD2, fy2
			character(len=15), intent(out)	:: result
			
			
			if (p2 == fU2) then
			
				result = "autodeleted"
				
			else if (p2 == fU) then
			
				result = fU_prime
				
			else if (p2 == fU_prime) then
			
				result = fU
				
			else if (p2 == fD2) then
			
				result = fy2
				
			else if (p2 == fy2) then
			
				result = fD2
				
			else
			
				result = "null"
				
			end if
			
		end subroutine reduce_double
		
	!______________________________________________________________________	
		
		subroutine reduce_axis (p2, fx, fx_prime, fx2, fR, fR_prime, fL, fL_prime, result)

			character(len=2), intent(in)	:: p2, fx, fx_prime, fx2, fR, fR_prime, fL, fL_prime
			character(len=15), intent(out)	:: result
			
			
			if (p2 == fx) then
			
				result = fx2
				
			else if (p2 == fx_prime) then
			
				result = "autodeleted"
				
			else if (p2 == fx2) then
			
				result = fx_prime
				
			else if (p2 == fR_prime) then
			
				result = fL_prime
				
			else if (p2 == fL) then
			
				result = fR
				
			else
			
				result = "null"
				
			end if
			
		end subroutine reduce_axis
		
	!______________________________________________________________________

		subroutine reduce_axis_prime (p2, fx_prime, fx, fx2, fR, fR_prime, fL, fL_prime, result)

			character(len=2), intent(in)	:: p2, fx, fx_prime, fx2, fR, fR_prime, fL, fL_prime
			character(len=15), intent(out)	:: result
			
			
			if (p2 == fx) then
			
				result = "autodeleted"
				
			else if (p2 == fx_prime) then
			
				result = fx2
				
			else if (p2 == fx2) then
			
				result = fx
				
			else if (p2 == fR) then
			
				result = fL
				
			else if (p2 == fL_prime) then
			
				result = fR_prime
				
			else
			
				result = "null"
				
			end if
			
		end subroutine reduce_axis_prime
		
	!______________________________________________________________________

		subroutine reduce_axis_double (p2, fx2, fx, fx_prime, fR2, fL2, result)

			character(len=2), intent(in)	:: p2, fx, fx_prime, fx2, fR2, fL2
			character(len=15), intent(out)	:: result
			
			
			if (p2 == fx) then
			
				result = fx_prime
				
			else if (p2 == fx_prime) then
			
				result = fx
				
			else if (p2 == fx2) then
			
				result = "autodeleted"
				
			else if (p2 == fR2) then
			
				result = fL2
				
			else if (p2 == fL2) then
			
				result = fR2
				
			else
			
				result = "null"
				
			end if
			
		end subroutine reduce_axis_double
		
		
	
end module module_1	